@@ -1,4 +1,4 @@
-/* $Id: thunar-vfs-io-local-xfer.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2007 Benedikt Meurer <benny@xfce.org>
  *
@@ -252,6 +252,11 @@ tvilx_copy_regular (const gchar                   *source_absolute_path,
   else
 #endif /* !HAVE_MMAP */
     {
+#ifdef HAVE_POSIX_FADVISE
+      /* tell the system that the data will be read sequentially */
+      posix_fadvise (source_fd, 0, 0, POSIX_FADV_SEQUENTIAL);
+#endif
+
       /* allocate the transfer buffer */
       bufsize = 8 * source_statb->st_blksize;
       buffer = g_new (gchar, bufsize);
