@@ -1,4 +1,4 @@
-/* $Id: thunar-vfs-mime-database.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -1122,11 +1122,20 @@ thunar_vfs_mime_database_get_info_for_file (ThunarVfsMimeDatabase *database,
             }
 #elif defined(HAVE_FGETXATTR)
           /* check for valid mime type stored in the extattr (Linux) */
+#if defined(__APPLE__) && defined(__MACH__)
+          nbytes = fgetxattr (fd, "user.mime_type", NULL, 0, 0, 0);
+#else
           nbytes = fgetxattr (fd, "user.mime_type", NULL, 0);
+#endif
           if (G_UNLIKELY (nbytes >= 3))
             {
               buffer = g_newa (gchar, nbytes + 1);
+#if defined(__APPLE__) && defined(__MACH__)
+              nbytes = fgetxattr (fd, "user.mime_type", buffer, nbytes, 0, 0);
+#else
               nbytes = fgetxattr (fd, "user.mime_type", buffer, nbytes);
+#endif
+
               if (G_LIKELY (nbytes >= 3))
                 {
                   buffer[nbytes] = '\0';
@@ -1566,7 +1575,7 @@ thunar_vfs_mime_database_set_default_application (ThunarVfsMimeDatabase    *data
           if (G_UNLIKELY (g_rename (path, absolute_path) < 0))
             {
               /* tell the caller that we failed */
-              g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), g_strerror (errno));
+              g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), "%s", g_strerror (errno));
               succeed = FALSE;
 
               /* be sure to remove the temporary file */
@@ -1575,7 +1584,7 @@ thunar_vfs_mime_database_set_default_application (ThunarVfsMimeDatabase    *data
         }
       else
         {
-          g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), g_strerror (errno));
+          g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), "%s", g_strerror (errno));
           succeed = FALSE;
         }
       g_free (path);
@@ -1643,7 +1652,7 @@ thunar_vfs_mime_database_add_application (ThunarVfsMimeDatabase *database,
   fp = fopen (file, "w");
   if (G_UNLIKELY (fp == NULL))
     {
-      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), g_strerror (errno));
+      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno), "%s", g_strerror (errno));
     }
   else
     {
@@ -1718,7 +1727,7 @@ thunar_vfs_mime_database_remove_application (ThunarVfsMimeDatabase    *database,
   /* verify that the application is usercreated */
   if (!thunar_vfs_mime_application_is_usercreated (application))
     {
-      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_INVAL, g_strerror (EINVAL));
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_INVAL, "%s", g_strerror (EINVAL));
       return FALSE;
     }
 
@@ -1729,7 +1738,7 @@ thunar_vfs_mime_database_remove_application (ThunarVfsMimeDatabase    *database,
   directory = xfce_resource_save_location (XFCE_RESOURCE_DATA, "applications/", TRUE);
   if (G_UNLIKELY (directory == NULL))
     {
-      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_NOTDIR, g_strerror (ENOTDIR));
+      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_NOTDIR, "%s", g_strerror (ENOTDIR));
       return FALSE;
     }
 
