@@ -1,4 +1,4 @@
-/* $Id: thunar-file.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2007 Benedikt Meurer <benny@xfce.org>
  *
@@ -69,14 +69,6 @@
 
 
 
-/* Property identifiers */
-enum
-{
-  PROP_0,
-  PROP_DISPLAY_NAME,
-  PROP_SPECIAL_NAME,
-};
-
 /* Signal identifiers */
 enum
 {
@@ -90,10 +82,6 @@ static void               thunar_file_class_init               (ThunarFileClass
 static void               thunar_file_info_init                (ThunarxFileInfoIface   *iface);
 static void               thunar_file_dispose                  (GObject                *object);
 static void               thunar_file_finalize                 (GObject                *object);
-static void               thunar_file_get_property             (GObject                *object,
-                                                                guint                   prop_id,
-                                                                GValue                 *value,
-                                                                GParamSpec             *pspec);
 static gchar             *thunar_file_info_get_name            (ThunarxFileInfo        *file_info);
 static gchar             *thunar_file_info_get_uri             (ThunarxFileInfo        *file_info);
 static gchar             *thunar_file_info_get_parent_uri      (ThunarxFileInfo        *file_info);
@@ -233,21 +221,6 @@ thunar_file_class_init (ThunarFileClass *klass)
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->dispose = thunar_file_dispose;
   gobject_class->finalize = thunar_file_finalize;
-  gobject_class->get_property = thunar_file_get_property;
-
-  /**
-   * ThunarFile::display-name:
-   *
-   * The file's display name, see thunar_file_get_display_name()
-   * for details.
-   **/
-  g_object_class_install_property (gobject_class,
-                                   PROP_DISPLAY_NAME,
-                                   g_param_spec_string ("display-name",
-                                                        "display-name",
-                                                        "display-name",
-                                                        NULL,
-                                                        EXO_PARAM_READABLE));
 
   /**
    * ThunarFile::destroy:
@@ -264,7 +237,6 @@ thunar_file_class_init (ThunarFileClass *klass)
                   NULL, NULL,
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
-
 }
 
 
@@ -333,28 +305,6 @@ thunar_file_finalize (GObject *object)
 
 
 
-static void
-thunar_file_get_property (GObject    *object,
-                          guint       prop_id,
-                          GValue     *value,
-                          GParamSpec *pspec)
-{
-  ThunarFile *file = THUNAR_FILE (object);
-
-  switch (prop_id)
-    {
-    case PROP_DISPLAY_NAME:
-      g_value_set_static_string (value, thunar_file_get_display_name (file));
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
-
-
 static gchar*
 thunar_file_info_get_name (ThunarxFileInfo *file_info)
 {
@@ -472,9 +422,6 @@ thunar_file_info_changed (ThunarxFileInfo *file_info)
    */
   thunar_file_set_thumb_state (THUNAR_FILE (file_info), THUNAR_FILE_THUMB_STATE_UNKNOWN);
 
-  /* notify about changes of the display-name property */
-  g_object_notify (G_OBJECT (file_info), "display-name");
-
   /* tell the file monitor that this file changed */
   thunar_file_monitor_file_changed (THUNAR_FILE (file_info));
 }
@@ -1621,6 +1568,33 @@ thunar_file_set_custom_icon (ThunarFile  *file,
 }
 
 
+/**
+ * thunar_file_is_desktop:
+ * @file : a #ThunarFile.
+ *
+ * Checks whether @file refers to the users desktop directory.
+ *
+ * Return value: %TRUE if @file is the users desktop directory.
+ **/
+gboolean
+thunar_file_is_desktop (const ThunarFile *file)
+{
+#if GLIB_CHECK_VERSION(2,14,0)
+  gchar file_path[THUNAR_VFS_PATH_MAXSTRLEN];
+
+  file_path[0] = '\0';
+  thunar_vfs_path_to_string (thunar_file_get_path (file), file_path,
+                             sizeof (file_path), NULL);
+
+  /* g_get_user_special_dir () always returns something for the desktop */
+  return exo_str_is_equal (file_path, g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
+#else /* GLIB_CHECK_VERSION(2,14,0) */
+return (!thunar_vfs_path_is_root (thunar_file_get_path (file))
+        && thunar_vfs_path_is_home (thunar_vfs_path_get_parent (thunar_file_get_path (file)))
+        && exo_str_is_equal (thunar_file_get_display_name (file), "Desktop"));
+#endif /* GLIB_CHECK_VERSION(2,14,0) */
+}
+
 
 /**
  * thunar_file_get_icon_name:
