@@ -1,4 +1,4 @@
-/* $Id: thunar-list-model.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2004-2007 Benedikt Meurer <benny@xfce.org>
  *
@@ -208,7 +208,7 @@ struct _ThunarListModel
    * generated by another model.
    */
 #ifndef NDEBUG
-  guint          stamp;
+  gint           stamp;
 #endif
 
   gint            nrows;
