@@ -1,4 +1,4 @@
-/* $Id: thunar-tree-view.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -115,10 +115,13 @@ static void             thunar_tree_view_row_activated              (GtkTreeView
 static gboolean         thunar_tree_view_test_expand_row            (GtkTreeView          *tree_view,
                                                                      GtkTreeIter          *iter,
                                                                      GtkTreePath          *path);
+static void             thunar_tree_view_row_collapsed              (GtkTreeView          *tree_view,
+                                                                     GtkTreeIter          *iter,
+                                                                     GtkTreePath          *path);
 static gboolean         thunar_tree_view_delete_selected_files      (ThunarTreeView       *view);
 static void             thunar_tree_view_context_menu               (ThunarTreeView       *view,
                                                                      GdkEventButton       *event,
-                                                                     GtkTreeModel         *filter,
+                                                                     GtkTreeModel         *model,
                                                                      GtkTreeIter          *iter);
 static GdkDragAction    thunar_tree_view_get_dest_actions           (ThunarTreeView       *view,
                                                                      GdkDragContext       *context,
@@ -136,6 +139,7 @@ static void             thunar_tree_view_action_copy                (ThunarTreeV
 static void             thunar_tree_view_action_create_folder       (ThunarTreeView       *view);
 static void             thunar_tree_view_action_cut                 (ThunarTreeView       *view);
 static void             thunar_tree_view_action_delete              (ThunarTreeView       *view);
+static void             thunar_tree_view_action_rename              (ThunarTreeView       *view);
 static void             thunar_tree_view_action_eject               (ThunarTreeView       *view);
 static void             thunar_tree_view_action_empty_trash         (ThunarTreeView       *view);
 static gboolean         thunar_tree_view_action_mount               (ThunarTreeView       *view);
@@ -148,8 +152,8 @@ static GClosure        *thunar_tree_view_new_files_closure          (ThunarTreeV
 static void             thunar_tree_view_new_files                  (ThunarVfsJob         *job,
                                                                      GList                *path_list,
                                                                      ThunarTreeView       *view);
-static gboolean         thunar_tree_view_filter_visible_func        (GtkTreeModel         *filter,
-                                                                     GtkTreeIter          *iter,
+static gboolean         thunar_tree_view_visible_func               (ThunarTreeModel      *model,
+                                                                     ThunarFile           *file,
                                                                      gpointer              user_data);
 static gboolean         thunar_tree_view_selection_func             (GtkTreeSelection     *selection,
                                                                      GtkTreeModel         *model,
@@ -181,6 +185,8 @@ struct _ThunarTreeView
   GtkCellRenderer        *icon_renderer;
   ThunarFile             *current_directory;
   ThunarTreeModel        *model;
+  
+  ThunarxProviderFactory *provider_factory;
 
   /* whether to display hidden/backup files */
   guint                   show_hidden : 1;
@@ -299,6 +305,7 @@ thunar_tree_view_class_init (ThunarTreeViewClass *klass)
   gtktree_view_class = GTK_TREE_VIEW_CLASS (klass);
   gtktree_view_class->row_activated = thunar_tree_view_row_activated;
   gtktree_view_class->test_expand_row = thunar_tree_view_test_expand_row;
+  gtktree_view_class->row_collapsed = thunar_tree_view_row_collapsed;
 
   klass->delete_selected_files = thunar_tree_view_delete_selected_files;
 
@@ -361,13 +368,15 @@ thunar_tree_view_init (ThunarTreeView *view)
   GtkTreeViewColumn *column;
   GtkTreeSelection  *selection;
   GtkCellRenderer   *renderer;
-  GtkTreeModel      *filter;
 
   /* initialize the view internals */
   view->cursor_idle_id = -1;
   view->drag_scroll_timer_id = -1;
   view->expand_timer_id = -1;
 
+  /* grab a reference on the provider factory */
+  view->provider_factory = thunarx_provider_factory_get_default ();
+
   /* grab a reference on the preferences; be sure to redraw the view
    * whenever the "tree-icon-emblems" preference changes.
    */
@@ -376,12 +385,8 @@ thunar_tree_view_init (ThunarTreeView *view)
 
   /* connect to the default tree model */
   view->model = thunar_tree_model_get_default ();
-
-  /* setup the filter for the tree view */
-  filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (view->model), NULL);
-  gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter), thunar_tree_view_filter_visible_func, view, NULL);
-  gtk_tree_view_set_model (GTK_TREE_VIEW (view), filter);
-  g_object_unref (G_OBJECT (filter));
+  thunar_tree_model_set_visible_func (view->model, thunar_tree_view_visible_func, view);
+  gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (view->model));
 
   /* configure the tree view */
   gtk_tree_view_set_enable_search (GTK_TREE_VIEW (view), FALSE);
@@ -444,6 +449,9 @@ thunar_tree_view_finalize (GObject *object)
   /* release drop path list (if drag_leave wasn't called) */
   thunar_vfs_path_list_free (view->drop_path_list);
 
+  /* release the provider factory */
+  g_object_unref (G_OBJECT (view->provider_factory));
+
   /* disconnect the queue resize signal handler */
 #if GTK_CHECK_VERSION(2,8,0)
   g_signal_handler_disconnect (G_OBJECT (view->preferences), view->queue_resize_signal_id);
@@ -543,7 +551,7 @@ thunar_tree_view_set_current_directory (ThunarNavigator *navigator,
                                         ThunarFile      *current_directory)
 {
   ThunarTreeView *view = THUNAR_TREE_VIEW (navigator);
-  GtkTreeModel   *filter;
+  ThunarFile     *file, *file_parent;
 
   /* check if we already use that directory */
   if (G_UNLIKELY (view->current_directory == current_directory))
@@ -562,11 +570,30 @@ thunar_tree_view_set_current_directory (ThunarNavigator *navigator,
       /* take a reference on the directory */
       g_object_ref (G_OBJECT (current_directory));
 
-      /* update the filter if the new current directory is hidden */
-      if (!view->show_hidden && thunar_file_is_hidden (current_directory))
+      /* update the filter if the new current directory, or one of it's parents, is hidden */
+      if (!view->show_hidden)
         {
-          filter = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
-          gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (filter));
+          /* look if the file or one of it's parents is hidden */
+          for (file = g_object_ref (G_OBJECT (current_directory)); file != NULL; file = file_parent)
+            {
+              /* check if this file is hidden */
+              if (thunar_file_is_hidden (file))
+                {
+                  /* update the filter */
+                  thunar_tree_model_refilter (view->model);
+
+                  /* release the file */
+                  g_object_unref (G_OBJECT (file));
+
+                  break;
+                }
+
+              /* get the file parent */
+              file_parent = thunar_file_get_parent (file, NULL);
+
+              /* release the file */
+              g_object_unref (G_OBJECT (file));
+            }
         }
 
       /* schedule an idle source to set the cursor to the current directory */
@@ -624,7 +651,6 @@ thunar_tree_view_button_press_event (GtkWidget      *widget,
                                      GdkEventButton *event)
 {
   ThunarTreeView *view = THUNAR_TREE_VIEW (widget);
-  GtkTreeModel   *filter;
   GtkTreePath    *path;
   GtkTreeIter     iter;
   gboolean        result;
@@ -646,11 +672,10 @@ thunar_tree_view_button_press_event (GtkWidget      *widget,
       if (G_UNLIKELY (event->button == 3 && event->type == GDK_BUTTON_PRESS))
         {
           /* determine the iterator for the path */
-          filter = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
-          if (gtk_tree_model_get_iter (filter, &iter, path))
+          if (gtk_tree_model_get_iter (GTK_TREE_MODEL (view->model), &iter, path))
             {
               /* popup the context menu */
-              thunar_tree_view_context_menu (view, event, filter, &iter);
+              thunar_tree_view_context_menu (view, event, GTK_TREE_MODEL (view->model), &iter);
 
               /* we effectively handled the event */
               result = TRUE;
@@ -679,7 +704,7 @@ thunar_tree_view_button_release_event (GtkWidget      *widget,
   ThunarTreeView *view = THUNAR_TREE_VIEW (widget);
 
   /* check if we have an event matching the pressed button state */
-  if (G_LIKELY (view->pressed_button == event->button))
+  if (G_LIKELY (view->pressed_button == (gint) event->button))
     {
       /* check if we should simply open or open in new window */
       if (G_LIKELY (event->button == 1))
@@ -769,7 +794,7 @@ thunar_tree_view_drag_drop (GtkWidget      *widget,
 
   /* determine the drop target */
   target = gtk_drag_dest_find_target (widget, context, NULL);
-  if (G_LIKELY (target == gdk_atom_intern ("text/uri-list", FALSE)))
+  if (G_LIKELY (target == gdk_atom_intern_static_string ("text/uri-list")))
     {
       /* set state so the drag-data-received handler
        * knows that this is really a drop this time.
@@ -802,7 +827,7 @@ thunar_tree_view_drag_motion (GtkWidget      *widget,
 
   /* determine the drop target */
   target = gtk_drag_dest_find_target (widget, context, NULL);
-  if (G_UNLIKELY (target != gdk_atom_intern ("text/uri-list", FALSE)))
+  if (G_UNLIKELY (target != gdk_atom_intern_static_string ("text/uri-list")))
     {
       /* reset the "drop-file" of the icon renderer */
       g_object_set (G_OBJECT (view->icon_renderer), "drop-file", NULL, NULL);
@@ -875,15 +900,15 @@ thunar_tree_view_popup_menu (GtkWidget *widget)
 {
   GtkTreeSelection *selection;
   ThunarTreeView   *view = THUNAR_TREE_VIEW (widget);
-  GtkTreeModel     *filter;
+  GtkTreeModel     *model;
   GtkTreeIter       iter;
 
   /* determine the selected row */
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
-  if (gtk_tree_selection_get_selected (selection, &filter, &iter))
+  if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
       /* popup the context menu */
-      thunar_tree_view_context_menu (view, NULL, filter, &iter);
+      thunar_tree_view_context_menu (view, NULL, model, &iter);
       return TRUE;
     }
   else if (GTK_WIDGET_CLASS (thunar_tree_view_parent_class)->popup_menu != NULL)
@@ -927,14 +952,12 @@ thunar_tree_view_test_expand_row (GtkTreeView *tree_view,
 {
   ThunarVfsVolume *volume;
   ThunarTreeView  *view = THUNAR_TREE_VIEW (tree_view);
-  GtkTreeModel    *filter;
   GtkWidget       *window;
   gboolean         expandable = TRUE;
   GError          *error = NULL;
 
   /* determine the volume for the iterator */
-  filter = gtk_tree_view_get_model (tree_view);
-  gtk_tree_model_get (filter, iter, THUNAR_TREE_MODEL_COLUMN_VOLUME, &volume, -1);
+  gtk_tree_model_get (GTK_TREE_MODEL (view->model), iter, THUNAR_TREE_MODEL_COLUMN_VOLUME, &volume, -1);
 
   /* check if we have a volume */
   if (G_UNLIKELY (volume != NULL))
@@ -968,11 +991,36 @@ thunar_tree_view_test_expand_row (GtkTreeView *tree_view,
 
 
 
+static void
+thunar_tree_view_row_collapsed (GtkTreeView *tree_view,
+                                GtkTreeIter *iter,
+                                GtkTreePath *path)
+{
+  /* schedule a cleanup of the tree model */
+  thunar_tree_model_cleanup (THUNAR_TREE_VIEW (tree_view)->model);
+}
+
+
+
 static gboolean
 thunar_tree_view_delete_selected_files (ThunarTreeView *view)
 {
+  GtkAccelKey     key;
+  GdkModifierType state;
+  gboolean        permanently;
+
   _thunar_return_val_if_fail (THUNAR_IS_TREE_VIEW (view), FALSE);
 
+  /* if this looks like a permanently delete */
+  permanently = (gtk_get_current_event_state (&state) && (state & GDK_SHIFT_MASK) != 0);
+
+  /* check if the user defined a custom accelerator and is not holding the
+   * shift button. if he or she has, we don't response to the predefined key 
+   * bindings (bug #4173) */
+  if (gtk_accel_map_lookup_entry ("<Actions>/ThunarStandardView/delete", &key)
+      && key.accel_key != 0 && key.accel_mods != 0 && permanently == FALSE)
+    return FALSE;
+
   /* ask the user whether to delete the folder... */
   thunar_tree_view_action_delete (view);
 
@@ -985,7 +1033,7 @@ thunar_tree_view_delete_selected_files (ThunarTreeView *view)
 static void
 thunar_tree_view_context_menu (ThunarTreeView *view,
                                GdkEventButton *event,
-                               GtkTreeModel   *filter,
+                               GtkTreeModel   *model,
                                GtkTreeIter    *iter)
 {
   ThunarVfsVolume *volume;
@@ -994,13 +1042,16 @@ thunar_tree_view_context_menu (ThunarTreeView *view,
   GtkWidget       *image;
   GtkWidget       *menu;
   GtkWidget       *item;
+  GtkWidget       *window;
+  GList           *providers, *lp;
+  GList           *actions = NULL, *tmp;
 
   /* verify that we're connected to the clipboard manager */
   if (G_UNLIKELY (view->clipboard == NULL))
     return;
 
   /* determine the file and volume for the given iter */
-  gtk_tree_model_get (filter, iter,
+  gtk_tree_model_get (model, iter,
                       THUNAR_TREE_MODEL_COLUMN_FILE, &file,
                       THUNAR_TREE_MODEL_COLUMN_VOLUME, &volume,
                       -1);
@@ -1041,13 +1092,12 @@ thunar_tree_view_context_menu (ThunarTreeView *view,
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
       gtk_widget_show (item);
 
-      /* check if the volume is a disc */
-      if (thunar_vfs_volume_is_disc (volume))
+      /* check if the volume is ejectable */
+      if (thunar_vfs_volume_is_ejectable (volume))
         {
           /* append the "Eject Volume" menu action */
           item = gtk_image_menu_item_new_with_mnemonic (_("E_ject Volume"));
           g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_tree_view_action_eject), view);
-          gtk_widget_set_sensitive (item, thunar_vfs_volume_is_ejectable (volume));
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
           gtk_widget_show (item);
         }
@@ -1146,7 +1196,7 @@ thunar_tree_view_context_menu (ThunarTreeView *view,
       gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
       gtk_widget_show (image);
 
-      /* "Delete" doesn't make much sense for volumes */
+      /* "Delete" and "Rename" don't make much sense for volumes */
       if (G_LIKELY (volume == NULL))
         {
           /* determine the parent file (required to determine "Delete" sensitivity) */
@@ -1167,12 +1217,72 @@ thunar_tree_view_context_menu (ThunarTreeView *view,
           /* cleanup */
           if (G_LIKELY (parent_file != NULL))
             g_object_unref (G_OBJECT (parent_file));
+
+          /* don't show the "Rename" action in the trash */
+          if (G_LIKELY (!thunar_file_is_trashed (file)))
+            {
+              /* append a separator item */
+              item = gtk_separator_menu_item_new ();
+              gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+              gtk_widget_show (item);
+
+              /* append the "Rename" menu action */
+              item = gtk_image_menu_item_new_with_mnemonic (_("_Rename..."));
+              g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_tree_view_action_rename), view);
+              gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+              gtk_widget_set_sensitive (item, thunar_file_is_writable (file));
+              gtk_widget_show (item);
+            }
         }
 
       /* append a separator item */
       item = gtk_separator_menu_item_new ();
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
       gtk_widget_show (item);
+      
+      /* add the providers menu for non-trashed items */
+      if (G_LIKELY (!thunar_file_is_trashed (file)))
+        {
+          /* load the menu providers from the provider factory */
+          providers = thunarx_provider_factory_list_providers (view->provider_factory, THUNARX_TYPE_MENU_PROVIDER);
+          if (G_LIKELY (providers != NULL))
+            {
+              /* determine the toplevel window we belong to */
+              window = gtk_widget_get_toplevel (GTK_WIDGET (view));
+              
+              /* load the actions offered by the menu providers */
+              for (lp = providers; lp != NULL; lp = lp->next)
+                {
+                  tmp = thunarx_menu_provider_get_folder_actions (lp->data, window, THUNARX_FILE_INFO (file));
+                  actions = g_list_concat (actions, tmp);
+                  g_object_unref (G_OBJECT (lp->data));
+                }
+              g_list_free (providers);
+
+              /* add the actions to the menu */
+              for (lp = actions; lp != NULL; lp = lp->next)
+                {
+                  item = gtk_action_create_menu_item (GTK_ACTION (lp->data));
+                  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+                  gtk_widget_show (item);
+
+                  /* release the reference on the action */
+                  g_object_unref (G_OBJECT (lp->data));
+                }
+
+              /* add a separator to the end of the menu */
+              if (G_LIKELY (lp != actions))
+                {
+                  /* append a menu separator */
+                  item = gtk_separator_menu_item_new ();
+                  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+                  gtk_widget_show (item);
+                }
+
+              /* cleanup */
+              g_list_free (actions);
+            }
+        }
     }
 
   /* append the "Properties" menu action */
@@ -1210,7 +1320,6 @@ thunar_tree_view_get_dest_actions (ThunarTreeView *view,
 {
   GdkDragAction actions = 0;
   GdkDragAction action = 0;
-  GtkTreeModel *filter;
   GtkTreePath  *path = NULL;
   GtkTreeIter   iter;
   ThunarFile   *file = NULL;
@@ -1222,14 +1331,11 @@ thunar_tree_view_get_dest_actions (ThunarTreeView *view,
   /* determine the path for x/y */
   if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (view), x, y, &path, NULL, NULL, NULL))
     {
-      /* determine the tree model filter */
-      filter = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
-
       /* determine the iter for the given path */
-      if (gtk_tree_model_get_iter (filter, &iter, path))
+      if (gtk_tree_model_get_iter (GTK_TREE_MODEL (view->model), &iter, path))
         {
           /* determine the file for the given path */
-          gtk_tree_model_get (filter, &iter, THUNAR_TREE_MODEL_COLUMN_FILE, &file, -1);
+          gtk_tree_model_get (GTK_TREE_MODEL (view->model), &iter, THUNAR_TREE_MODEL_COLUMN_FILE, &file, -1);
           if (G_LIKELY (file != NULL))
             {
               /* check if the file accepts the drop */
@@ -1289,7 +1395,7 @@ thunar_tree_view_find_closest_ancestor (ThunarTreeView *view,
                                         GtkTreePath   **ancestor_return,
                                         gboolean       *exact_return)
 {
-  GtkTreeModel *filter;
+  GtkTreeModel *model = GTK_TREE_MODEL (view->model);
   GtkTreePath  *child_path;
   GtkTreeIter   child_iter;
   GtkTreeIter   iter;
@@ -1297,15 +1403,14 @@ thunar_tree_view_find_closest_ancestor (ThunarTreeView *view,
   gboolean      found = FALSE;
 
   /* determine the iter for the current path */
-  filter = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
-  if (!gtk_tree_model_get_iter (filter, &iter, path))
+  if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (view->model), &iter, path))
     return FALSE;
 
   /* process all items at this level */
   do
     {
       /* check if the file for iter is an ancestor of the current directory */
-      gtk_tree_model_get (filter, &iter, THUNAR_TREE_MODEL_COLUMN_FILE, &file, -1);
+      gtk_tree_model_get (model, &iter, THUNAR_TREE_MODEL_COLUMN_FILE, &file, -1);
       if (G_UNLIKELY (file == NULL))
         continue;
 
@@ -1313,16 +1418,16 @@ thunar_tree_view_find_closest_ancestor (ThunarTreeView *view,
       if (G_UNLIKELY (file == view->current_directory))
         {
           /* we found the very best ancestor iter! */
-          *ancestor_return = gtk_tree_model_get_path (filter, &iter);
+          *ancestor_return = gtk_tree_model_get_path (model, &iter);
           *exact_return = TRUE;
           found = TRUE;
         }
       else if (thunar_file_is_ancestor (view->current_directory, file))
         {
           /* check if we can find an even better ancestor below this node */
-          if (gtk_tree_model_iter_children (filter, &child_iter, &iter))
+          if (gtk_tree_model_iter_children (model, &child_iter, &iter))
             {
-              child_path = gtk_tree_model_get_path (filter, &child_iter);
+              child_path = gtk_tree_model_get_path (model, &child_iter);
               found = thunar_tree_view_find_closest_ancestor (view, child_path, ancestor_return, exact_return);
               gtk_tree_path_free (child_path);
             }
@@ -1331,7 +1436,7 @@ thunar_tree_view_find_closest_ancestor (ThunarTreeView *view,
           if (G_UNLIKELY (!found))
             {
               /* we found an ancestor, not an exact match */
-              *ancestor_return = gtk_tree_model_get_path (filter, &iter);
+              *ancestor_return = gtk_tree_model_get_path (model, &iter);
               *exact_return = FALSE;
               found = TRUE;
             }
@@ -1340,7 +1445,7 @@ thunar_tree_view_find_closest_ancestor (ThunarTreeView *view,
       /* release the file reference */
       g_object_unref (G_OBJECT (file));
     }
-  while (!found && gtk_tree_model_iter_next (filter, &iter));
+  while (!found && gtk_tree_model_iter_next (model, &iter));
 
   return found;
 }
@@ -1351,14 +1456,14 @@ static ThunarFile*
 thunar_tree_view_get_selected_file (ThunarTreeView *view)
 {
   GtkTreeSelection *selection;
-  GtkTreeModel     *filter;
+  GtkTreeModel     *model;
   GtkTreeIter       iter;
   ThunarFile       *file = NULL;
 
   /* determine file for the selected row */
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
-  if (gtk_tree_selection_get_selected (selection, &filter, &iter))
-    gtk_tree_model_get (filter, &iter, THUNAR_TREE_MODEL_COLUMN_FILE, &file, -1);
+  if (gtk_tree_selection_get_selected (selection, &model, &iter))
+    gtk_tree_model_get (model, &iter, THUNAR_TREE_MODEL_COLUMN_FILE, &file, -1);
 
   return file;
 }
@@ -1370,13 +1475,13 @@ thunar_tree_view_get_selected_volume (ThunarTreeView *view)
 {
   GtkTreeSelection *selection;
   ThunarVfsVolume  *volume = NULL;
-  GtkTreeModel     *filter;
+  GtkTreeModel     *model;
   GtkTreeIter       iter;
 
   /* determine file for the selected row */
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
-  if (gtk_tree_selection_get_selected (selection, &filter, &iter))
-    gtk_tree_model_get (filter, &iter, THUNAR_TREE_MODEL_COLUMN_VOLUME, &volume, -1);
+  if (gtk_tree_selection_get_selected (selection, &model, &iter))
+    gtk_tree_model_get (model, &iter, THUNAR_TREE_MODEL_COLUMN_VOLUME, &volume, -1);
 
   return volume;
 }
@@ -1524,6 +1629,31 @@ thunar_tree_view_action_delete (ThunarTreeView *view)
 
 
 
+static void
+thunar_tree_view_action_rename (ThunarTreeView *view)
+{
+  ThunarFile *file;
+  GtkWidget  *window;
+
+  _thunar_return_if_fail (THUNAR_IS_TREE_VIEW (view));
+  
+  /* determine the selected file */
+  file = thunar_tree_view_get_selected_file (view);
+  if (G_LIKELY (file != NULL))
+    {  
+      /* get the toplevel window */
+      window = gtk_widget_get_toplevel (GTK_WIDGET (view));
+      
+      /* run the rename dialog */
+      thunar_dialogs_show_rename_file (GTK_WINDOW (window), file);
+      
+      /* release the file */
+      g_object_unref (G_OBJECT (file));
+    }
+}
+
+
+
 static void
 thunar_tree_view_action_eject (ThunarTreeView *view)
 {
@@ -1539,7 +1669,7 @@ thunar_tree_view_action_eject (ThunarTreeView *view)
     {
       /* determine the toplevel window */
       window = gtk_widget_get_toplevel (GTK_WIDGET (view));
-      
+
       /* try to eject the volume */
       if (!thunar_vfs_volume_eject (volume, window, &error))
         {
@@ -1790,28 +1920,22 @@ thunar_tree_view_new_files (ThunarVfsJob   *job,
 
 
 static gboolean
-thunar_tree_view_filter_visible_func (GtkTreeModel *filter,
-                                      GtkTreeIter  *iter,
-                                      gpointer      user_data)
+thunar_tree_view_visible_func (ThunarTreeModel *model,
+                               ThunarFile      *file,
+                               gpointer         user_data)
 {
   ThunarTreeView *view = THUNAR_TREE_VIEW (user_data);
-  ThunarFile     *file;
   gboolean        visible = TRUE;
 
+  _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
+  _thunar_return_val_if_fail (THUNAR_IS_TREE_MODEL (model), FALSE);
+
   /* if show_hidden is TRUE, nothing is filtered */
   if (G_LIKELY (!view->show_hidden))
     {
-      /* check if we should display this row */
-      gtk_tree_model_get (filter, iter, THUNAR_TREE_MODEL_COLUMN_FILE, &file, -1);
-      if (G_LIKELY (file != NULL))
-        {
-          /* we display all non-hidden file and hidden files that are ancestors of the current directory */
-          visible = !thunar_file_is_hidden (file) || (view->current_directory == file)
-                 || (view->current_directory != NULL && thunar_file_is_ancestor (view->current_directory, file));
-
-          /* release the reference on the file */
-          g_object_unref (G_OBJECT (file));
-        }
+      /* we display all non-hidden file and hidden files that are ancestors of the current directory */
+      visible = !thunar_file_is_hidden (file) || (view->current_directory == file)
+                || (view->current_directory != NULL && thunar_file_is_ancestor (view->current_directory, file));
     }
 
   return visible;
@@ -1821,7 +1945,7 @@ thunar_tree_view_filter_visible_func (GtkTreeModel *filter,
 
 static gboolean
 thunar_tree_view_selection_func (GtkTreeSelection *selection,
-                                 GtkTreeModel     *filter,
+                                 GtkTreeModel     *model,
                                  GtkTreePath      *path,
                                  gboolean          path_currently_selected,
                                  gpointer          user_data)
@@ -1836,10 +1960,10 @@ thunar_tree_view_selection_func (GtkTreeSelection *selection,
     return TRUE;
 
   /* determine the iterator for the path */
-  if (gtk_tree_model_get_iter (filter, &iter, path))
+  if (gtk_tree_model_get_iter (model, &iter, path))
     {
       /* determine the file for the iterator */
-      gtk_tree_model_get (filter, &iter, THUNAR_TREE_MODEL_COLUMN_FILE, &file, -1);
+      gtk_tree_model_get (model, &iter, THUNAR_TREE_MODEL_COLUMN_FILE, &file, -1);
       if (G_LIKELY (file != NULL))
         {
           /* rows with files can be selected */
@@ -1851,7 +1975,7 @@ thunar_tree_view_selection_func (GtkTreeSelection *selection,
       else
         {
           /* but maybe the row has a volume */
-          gtk_tree_model_get (filter, &iter, THUNAR_TREE_MODEL_COLUMN_VOLUME, &volume, -1);
+          gtk_tree_model_get (model, &iter, THUNAR_TREE_MODEL_COLUMN_VOLUME, &volume, -1);
           if (G_LIKELY (volume != NULL))
             {
               /* rows with volumes can also be selected */
@@ -2121,9 +2245,8 @@ void
 thunar_tree_view_set_show_hidden (ThunarTreeView *view,
                                   gboolean        show_hidden)
 {
-  GtkTreeModel *filter;
-
   _thunar_return_if_fail (THUNAR_IS_TREE_VIEW (view));
+  _thunar_return_if_fail (THUNAR_IS_TREE_MODEL (view->model));
 
   /* normalize the value */
   show_hidden = !!show_hidden;
@@ -2134,13 +2257,10 @@ thunar_tree_view_set_show_hidden (ThunarTreeView *view,
       /* apply the new setting */
       view->show_hidden = show_hidden;
 
-      /* update the filter */
-      filter = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
-      gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (filter));
+      /* update the model */
+      thunar_tree_model_refilter (view->model);
 
       /* notify listeners */
       g_object_notify (G_OBJECT (view), "show-hidden");
     }
 }
-
-
